% script per caricare i dati da ball-and-beam, creare delle strutture dati
% per il SysId toolbox e disegnare i grafici
% origine dei dati: database DaISy
% http://homes.esat.kuleuven.be/~smc/daisy/

clear all
close all hidden
clc

%% -------- carico i dati e disegno
load ballbeam.dat
U=ballbeam(:,1);
Y=ballbeam(:,2);
Ts = 0.10; % campionamento
time_vect = (0: numel(U)-1)* Ts; % istanti di tempo associati ai campioni

hf=figure('Name','Ball & beam - campioni');
axhnd1 = subplot(2,1,1);plot(time_vect, U); grid on;
xlabel('istanti di tempo [s]'); 
ylabel('angolo d''inclinazione dell''asta [rad]');

axhnd2 = subplot(2,1,2);plot(time_vect, Y); grid on;
xlabel('istanti di tempo [s]'); 
ylabel('posizione della palla [m]');

linkaxes([axhnd1 axhnd2],'x'); % cosi' sincronizzo lo zoom nei 2 grafici

%% ---------- pre-elaborazione dei dati: detrend
data_set = iddata(Y, U, Ts, 'OutputName','ball position',...
    'OutputUnit','m', 'InputName','beam rotation angle',...
    'InputUnit','rad','Tstart',0,'TimeUnit','seconds'); 
% oggetto iddata che memorizza tutti i dati
% NB:
% The "TimeUnit" property must be set to one of the following:
% 'nanoseconds', 'microseconds', 'milliseconds', 'seconds', 'minutes',
% 'hours', 'days', 'weeks', 'months', or 'years'.
%
[data_s_d T] = detrend(data_set); 
% rendo i dati a media nulla e tolgo eventuali trend

%% prima analisi: uso gli indici AIC, FPE, MDL e cerco un modello ARX

% -----------------------------------
% ordine di persistente eccitazione?
pe_n = pexcit(data_s_d);
messaggio = sprintf('L''ordine di persistente eccitazione vale almeno n= %d \n', pe_n);
h = msgbox(messaggio,'modal');
uiwait(h,120);
pause(3); close(hf);
% -----------------------------------

disp('--------------------------------------------------');
disp('NB: l''elaborazione puo'' durare diversi minuti!');
disp('--------------------------------------------------');

% per avere AIC, FPE, MDL procedo in questo modo: identifico tutti i
% modelli usando tutti i dati a disposizione e poi valuto gli indici,
% memorizzando il risultato per ciascun modello. Infine, esplorando le
% curve degli indici, determino i "vincitori"

complessitaARX = struc(1:15, 1:10, 1:8);
% voglio esplorare gli ARX per n_a da 1 a 15, n_b da 1 a 10, n_k da 1 a 8

% inizializzo i vettori dove memorizzare gli indici
AIC_ARXscores = zeros(size(complessitaARX,1),1);
FPE_ARXscores = zeros(size(AIC_ARXscores));
MDL_ARXscores = zeros(size(AIC_ARXscores));

% e la struttura che memorizza i modelli ARX
ARX_set = cell(size(AIC_ARXscores));

% -----------------------------------------------------------
tic
for ii=1: size(complessitaARX,1)
    modello_ARX = arx(data_s_d, complessitaARX(ii,:));
    ARX_set{ii} = modello_ARX;
    % identifico un modello ARX di ordine imposto dal vettore riga
    % complessitaARX(ii,:)
    % ATTENZIONE: codice NON efficiente
    % impiega diverso tempo ad eseguire le valutazioni richieste
    % (piu' di 6 minuti su un Core 2 Duo a 3 GHz)
    AIC_ARXscores(ii) = aic(modello_ARX);
    FPE_ARXscores(ii) = fpe(modello_ARX);
    MDL_ARXscores(ii) = mdl_score(modello_ARX);
    % disp(ii);
end % for ii
tempo_speso = toc;
fprintf(1, 'tempo speso per l''analisi dei modelli ARX %f [s]\n', tempo_speso);
% -----------------------------------------------------------

[aic_min,aic_id] = min(AIC_ARXscores);
[fpe_min,fpe_id] = min(FPE_ARXscores);
[mdl_min,mdl_id] = min(MDL_ARXscores);

aic_best_arx = ARX_set{aic_id};aic_best_arx.Name = 'ARX model with best AIC score';
fpe_best_arx = ARX_set{fpe_id};fpe_best_arx.Name = 'ARX model with best FPE score';
mdl_best_arx = ARX_set{mdl_id};mdl_best_arx.Name = 'ARX model with best MDL score';

hf1 = figure; set(hf1,'Visible','off');
disp('===================================');
disp('il miglior modello ARX secondo AIC:');
disp(aic_best_arx);
disp('------------');
fprintf(1,...
    'complessita'' del modello: n_a=%g, n_b=%g, n_k=%g \n',...
    complessitaARX(aic_id,:));

disp('===================================');
disp('il miglior modello ARX secondo FPE:');
disp(fpe_best_arx);
disp('------------');
fprintf(1,...
    'complessita'' del modello: n_a=%g, n_b=%g, n_k=%g \n',...
    complessitaARX(fpe_id,:));

disp('===================================');
disp('il miglior modello ARX secondo MDL:');
disp(mdl_best_arx);disp('------------');
fprintf(1,...
    'complessita'' del modello: n_a=%g, n_b=%g, n_k=%g \n',...
    complessitaARX(mdl_id,:));

% -----------------------------
set(hf1,'Visible','on')
compare(data_s_d, aic_best_arx, fpe_best_arx, mdl_best_arx, 1);
% confronto delle prestazioni come predittori ad 1 passo in avanti
% -----------------------------
% ed ora i test grafici sui residui
maxLag = floor(numel(data_s_d.InputData)/4);
Res_opt = residOptions('MaxLag', maxLag);
hfc = figure;resid(data_s_d, aic_best_arx, Res_opt);
set(hfc,'Name',[get(hfc,'Name'),' - model:', aic_best_arx.Name])

hfc = figure;resid(data_s_d, fpe_best_arx, Res_opt);
set(hfc,'Name',[get(hfc,'Name'),' - model:', fpe_best_arx.Name])

hfc = figure;resid(data_s_d, mdl_best_arx, Res_opt);
set(hfc,'Name',[get(hfc,'Name'),' - model:', mdl_best_arx.Name])

% -----------------------------
% ed ora i test di Anderson sui residui
confidence_alpha_value = 0.01;
H_AIC = AndersonWhitenessTest(aic_best_arx, data_s_d,...
                                confidence_alpha_value, true);
% test: si puo' negare che il residuo di predizione sia 
%       realizzazione di processo bianco?
% se H = 0 -> non si puo' negare
% se H = 1 -> ipotesi rigettata: la distribuzione NON e' quella desiderata,
%               i residui NON sono di un processo di rumore bianco

H_FPE = AndersonWhitenessTest(fpe_best_arx, data_s_d, ...
                                confidence_alpha_value, true);

H_MDL = AndersonWhitenessTest(mdl_best_arx, data_s_d, ...
                                confidence_alpha_value, true);

% NB quale modello usare?
% =====================================================================